/* $Id$

This file is part of gldbg
Copyright © 2009  Mikko Rasa, Mikkosoft Productions
Distributed under the GPL
*/

#ifndef COMMANDINTERPRETER_H_
#define COMMANDINTERPRETER_H_

#include <map>
#include <string>

class GlDbg;

class CommandInterpreter
{
private:
	struct Command
	{
		typedef void (CommandInterpreter::*Func)(const std::string &);

		Func func;
		std::string description;
		std::string help;

		Command();
		Command(Func, const std::string &);
		Command(Func, const std::string &, const std::string &);
	};

	typedef std::map<std::string, Command> CommandMap;

	GlDbg &gldbg;
	CommandMap commands;

public:
	CommandInterpreter(GlDbg &);
	void execute(const std::string &);

private:
	void cmd_help(const std::string &);
	void cmd_run(const std::string &);
	void cmd_continue(const std::string &);
	void cmd_signal(const std::string &);
	void cmd_kill(const std::string &);
	void cmd_exit(const std::string &);
	void cmd_trace(const std::string &);
	void cmd_profile(const std::string &);
	void cmd_state(const std::string &);
	void cmd_texture(const std::string &);
	void cmd_buffer(const std::string &);
};

#endif
